@@ -181,6 +181,7 @@ start_pinentry (ctrl_t ctrl)
   int no_close_list[3];
   int i;
   pth_event_t evt;
+  const char *tmpstr;
 
   evt = pth_event (PTH_EVENT_TIME, pth_timeout (LOCK_TIMEOUT, 0));
   if (!pth_mutex_acquire (&entry_lock, 0, evt))
@@ -213,12 +214,21 @@ start_pinentry (ctrl_t ctrl)
 
   if (!opt.pinentry_program || !*opt.pinentry_program)
     opt.pinentry_program = GNUPG_DEFAULT_PINENTRY;
+    pgmname = opt.pinentry_program;
   if ( !(pgmname = strrchr (opt.pinentry_program, '/')))
     pgmname = opt.pinentry_program;
   else
     pgmname++;
 
+  /* OS X needs the entire file name in argv[0], so that it can locate
+     the resource bundle.  For other systems we stick to the the usual
+     convention of supplying only the name of the program.  */
+#ifdef __APPLE__
+  argv[0] = opt.pinentry_program;
+#else /*!__APPLE__*/
   argv[0] = pgmname;
+#endif /*__APPLE__*/
+
   if (ctrl->display && !opt.keep_display)
     {
       argv[1] = "--display";
@@ -297,6 +307,30 @@ start_pinentry (ctrl_t ctrl)
       if (rc)
 	return unlock_pinentry (rc);
     }
+
+  
+  /* Tell the pinentry the name of a file it shall touch after having
+     messed with the tty.  This is optional and only supported by
+     newer pinentries and thus we do no error checking. */
+  tmpstr = opt.pinentry_touch_file;
+  if (tmpstr && !strcmp (tmpstr, "/dev/null"))
+    tmpstr = NULL;
+  else if (!tmpstr)
+    tmpstr = get_agent_socket_name ();
+  if (tmpstr)
+    {
+      char *optstr;
+      
+      if (asprintf (&optstr, "OPTION touch-file=%s", tmpstr ) < 0 )
+        ;
+      else
+        {
+          assuan_transact (entry_ctx, optstr, NULL, NULL, NULL, NULL, NULL,
+                           NULL);
+          free (optstr);
+        }
+    }
+
   return 0;
 }
 
